!**************************************************************************
!* Spatially Explicit Individual-Based Dynamic Global Vegetation Model    *
!* SEIB-DGVM ver. 2.50 (31 Dec 2009)                                      *
!*                                                                        *
!*   All rights are reserved by Dr. Hisashi SATO                          *
!*   @ Japan Agency for Marine-Earth Science and Technology (JAMSTEC)     *
!*                                                                        *
!*   Latest information and code using policy can be obtained at          *
!*   http://seib-dgvm.com/                                                *
!**************************************************************************

!*************************************************************************************************
! MAIN SIMULATION LOOP
!*************************************************************************************************
SUBROUTINE main_loop ( LAT, LON, &                        
           tmp_air, tmp_soil, cloud, prec, humid, wind, &
           ALT, Albedo_soil0, W_fi, W_wilt, W_sat, W_mat )

!_____________ Set variables
!Namespace
   USE data_structure
   USE time_counter
   USE vegi_status_current1
   USE vegi_status_current2
   USE grid_status_current1
   USE grid_status_current2
   implicit none
   
!Augments (Coordination)
   real,intent(IN):: LAT          !latitude  (degree)
   real,intent(IN):: LON          !longitude (degree)
   
!Augments (Climatic data)
   real,dimension(Day_in_Year)   ::tmp_air   !2m air temperature (Celcius)
   real,dimension(Day_in_Year,30)::tmp_soil  !soil temperature at 10 cm  depth (Celcius)
   real,dimension(Day_in_Year)   ::cloud     !total cloudiness (fraction)
   real,dimension(Day_in_Year)   ::prec      !precipitation (mm day-1)
   real,dimension(Day_in_Year)   ::humid     !specific humidity (kg kg-1)
   real,dimension(Day_in_Year)   ::wind      !wind velocity (m s-1)
   
!Augments (Location data)
   real,intent(IN):: ALT          !altitude  (m above MSL)
   real,intent(IN):: Albedo_soil0 !soil albedo
   real,intent(IN):: W_fi         !filed capacity   (m3/m3, 0.0 -> 1.0)
   real,intent(IN):: W_wilt       !wilting point    (m3/m3, 0.0 -> 1.0)
   real,intent(IN):: W_sat        !saturate point   (m3/m3, 0.0 -> 1.0)
   real,intent(IN):: W_mat        !matrix potential 
   
!Local variables
   integer i, j, k, p, no  !loop counter
   real    x, y, z      !temporal variables for general usage
   
!_____________ Intialize variables
!Reset variables
    Call init_value (W_sat, tmp_air(:), tmp_soil(:,1), prec(:))
   
!Read spinup files
   if (Flag_spinup_read) then
      open (File_no(1), file=Fn_spnin)
      Call spinup_in (File_no(1))
      close (File_no(1))
   endif
!_____________ Open output files
   open ( File_no( 1), file = 'output_log.txt'          )
   open ( File_no( 2), file = 'output.txt'              )
   open ( File_no( 3), file = 'output_climate.txt'      )
   open ( File_no( 4), file = 'output_air.txt'          )
   open ( File_no( 5), file = 'output_radiation.txt'    )
   open ( File_no( 6), file = 'output_netradiation.txt' )
   open ( File_no( 7), file = 'output_water.txt'        )
   open ( File_no( 8), file = 'output_grass.txt'        )
   open ( File_no( 9), file = 'output_lai.txt'          )
   open ( File_no(10), file = 'output_cflux.txt'        )
   open ( File_no(11), file = 'output_wflux.txt'        )
   open ( File_no(12), file = 'output_ld_vertical.txt'  )
   open ( File_no(13), file = 'output_annual.txt'       )
   open ( File_no(14), file = 'output_forest.txt'       )
   open ( File_no(15), file = 'output_biomass.txt'      )
   
!_____________ Starting point of the simulation loop
   if (Flag_spinup_read) then
      counter_begin = 1 + Spinup_year*Day_in_Year
      counter_end   = (Simulation_year+Spinup_year) * Day_in_Year
   else
      counter_begin = 1
      counter_end   = Simulation_year * Day_in_Year
   endif
   
   doy  = 0
   year = 1
   
Do counter = counter_begin, counter_end

!_____________ Daily update of field status
!Time counters1
   doy = doy + 1
   if (doy == Day_in_Year + 1) then
      doy   = 1
      year  = year + 1
   endif
   
!Time counters2
   dfl_fire = dfl_fire + 1
   
   Do p=1, PFT_no
      dfl_leaf_onset(p) = dfl_leaf_onset(p) + 1
      dfl_leaf_shed (p) = dfl_leaf_shed (p) + 1
   End do
   
!Climate statictics
   if (Logging==1) write (File_no(1),*) '    climate_stat'
   Call climate_stat (tmp_air(doy), &
        sum(tmp_soil(doy,1:5))/5.0, prec(doy))
   
!_____________ DAILY UPDATE OF METABOLIC STATUS
!Carbon Pool
   pool_c_RR (2:Day_in_Year) = pool_c_RR (1:Day_in_Year-1) 
   
   x = pool_litter_trunk + pool_litter_leaf + pool_litter_root + pool_litter_ag + pool_litter_bg + &
       pool_som_int + pool_som_slow
   
   y = 0.0
   Do no=1, Max_no
   if (tree_exist(no)) then
      y = y + mass_leaf(no) + mass_trunk(no) + mass_root(no) + mass_stock(no) + mass_available(no)
   endif
   End do
   
   z = sum(gmass_leaf(:,:)) + sum(gmass_root(:,:)) + sum(gmass_available(:,:)) + sum(gmass_stock(:,:))
   
   pool_c_RR (1) = (x + y + z) * C_in_drymass / Max_loc / Max_loc /1000.0 ! (g DM / forest) -> (Kg C / m2)
   
!Carbon Flux
   do p = 1, PFT_no
      npp_RunningRecord(p,2:Day_in_Year) = npp_RunningRecord(p,1:Day_in_Year-1)
      npp_RunningRecord(p,1)             = npp(p)
      
      gpp_RunningRecord(p,2:Day_in_Year) = gpp_RunningRecord(p,1:Day_in_Year-1)
      gpp_RunningRecord(p,1)             = gpp(p)
   end do
   
   gpp_daily_ind (:) = 0.0
   gpp           (:) = 0.0
   npp           (:) = 0.0
   
   flux_c_uptake_RR(2:Day_in_Year) = flux_c_uptake_RR(1:Day_in_Year-1) ; flux_c_uptake_RR(1) = 0.0
   flux_c_mnt_RR   (2:Day_in_Year) = flux_c_mnt_RR   (1:Day_in_Year-1) ; flux_c_mnt_RR   (1) = 0.0
   flux_c_gro_RR   (2:Day_in_Year) = flux_c_gro_RR   (1:Day_in_Year-1) ; flux_c_gro_RR   (1) = 0.0
   flux_c_htr_RR   (2:Day_in_Year) = flux_c_htr_RR   (1:Day_in_Year-1) ; flux_c_htr_RR   (1) = 0.0
   flux_c_fir_RR   (2:Day_in_Year) = flux_c_fir_RR   (1:Day_in_Year-1) ; flux_c_fir_RR   (1) = 0.0
   
   !Carbon flux for each organ (reset at the beggining of each day)
   resp_trunc(:) = 0.0
   resp_leaf (:) = 0.0
   resp_root (:) = 0.0
   resp_grass_ag = 0.0
   resp_grass_bg = 0.0
   
   !Litter flux (reset at the beggining of each day)
   flux_litter_trunk = 0.0
   flux_litter_leaf  = 0.0
   flux_litter_root  = 0.0
   flux_litter_ag    = 0.0
   flux_litter_bg    = 0.0
   
!LAI
   !annual maximum and average
   if (doy==1) then
      lai_max_wood   = 0.0
      lai_max_grass  = 0.0
      lai_mean_wood  = 0.0
      lai_mean_grass = 0.0
   endif
   
   !woody PFTs
   x = 0.0
   do no=1, Max_no
      if (tree_exist(no)) x = x + la(no)
   enddo
   x = x / Max_loc / Max_loc
   
   lai_max_wood  = max(x, lai_max_wood)
   lai_mean_wood = lai_mean_wood  + x / real(Day_in_Year)
   
   !grass PFTs
   x = sum(lai_grass(:,:)) / DivedG / DivedG !Mean grass LAI over grass layer
   lai_max_grass  = max(x, lai_max_grass)
   lai_mean_grass = lai_mean_grass + x / real(Day_in_Year)
   
!stat_water
   do p = 1, PFT_no
      x = 0.0
      y = 0.0
      
      j = max(1,RootDepth(p))
      do i=1, j
         x =     x + (pool_w(i)/Depth - W_wilt) / max(W_fi-W_wilt, 0.001) /real(j) !Method1
         y = max(y,  (pool_w(i)/Depth - W_wilt) / max(W_fi-W_wilt, 0.001) )        !Method2
      enddo
      
      if (Life_type(p)==2) then
         stat_water(p) = y !for larch
      else
         stat_water(p) = x !for other PFTs
      endif
      
      stat_water_RunningRecord(p,2:Day_in_Year) = stat_water_RunningRecord(p,1:Day_in_Year-1)
      stat_water_RunningRecord(p,1)             = stat_water(p)
   enddo
   
!Phenology
   do p = 1, PFT_no
      phenology_RunningRecord(p,2:Day_in_Year) = phenology_RunningRecord(p,1:Day_in_Year-1)
      phenology_RunningRecord(p,1)             = phenology(p)
   enddo
   
!Growth suppresss regulator
   if (doy==365) then
      flag_suppress(:) = 0
      do no=1, Max_no
         if ( tree_exist(no) ) then
         if ( mort_regu1(no)<10.0 .and. age(no)>3 ) then
            flag_suppress(no)=1
         endif
         endif
      enddo
   end if
   
!Mortality related
   if (doy==1) then
      mort_regu1(:) = 0.0 !NPP annual (g / individual)
      mort_regu2(:) = 0.0 !average leaf area of last year (m2/day) {update on: growth_wood}
      mort_regu4(:) = 0.0 !stem diameter increament in last year (m year-1)
   endif
   
!_____________ DAILY PSYSIOLOGICAL PROCESSES
   if (Logging==1) write (File_no(1),*) '    co2'
   Call co2_density (LAT)
   
   if (Logging==1) write (File_no(1),*) '    air'
   Call air (tmp_air(doy), humid(doy), wind(doy), ALT)
   
   if (Logging==1) write (File_no(1),*) '    radiation'
   Call radiation (LAT, cloud(doy))
   par_RunningRecord(2:Day_in_Year) = par_RunningRecord(1:Day_in_Year-1)
   par_RunningRecord(1)             = par
   
   if (Logging==1) write (File_no(1),*) '    light_condition: diffuse'
   Call diffused_radiation ()
   
   if (mod(doy,14)==1) then
      if (Logging==1) write (File_no(1),*) '    light_condition: direct'
      Call direct_radiation ()
      
      if (Logging==1) write (File_no(1),*) '    light_condition: grass'
      Call floor_radiation ()
   end if
   
   if (Logging==1) write (File_no(1),*) '    light_condition: interruption index'
   Call ir_index ()
   
!_____________ DAILY BIOLOGICAL PROCESSES
   if (Logging==1) write (File_no(1),*) '    photosynthesis_condition'
   Call photosynthesis_condition (tmp_air(doy))
   
   if (Logging==1) write (File_no(1),*) '    photosynthesis'
   Call photosynthesis ()
   
   if (Logging==1) write (File_no(1),*) '    lai_optimum'
   Call lai_optimum (tmp_air(doy), sum(tmp_soil(doy,1:5))/5.0 )
   
   if (Logging==1) write (File_no(1),*) '    maintenance_resp'
   Call maintenance_resp (tmp_air(doy), sum(tmp_soil(doy,1:5))/5.0 )
   
   if (Logging==1) write (File_no(1),*) '    turnover'
   Call turnover ()
   
   if (Logging==1) write (File_no(1),*) '    leaf_season'
   Call leaf_season (LAT, sum(tmp_soil(doy,1:5))/5.0 )
   
   if (Logging==1) write (File_no(1),*) '    growth_wood_daily'
   Call growth_wood ()
   
   if (Logging==1) write (File_no(1),*) '    growth_grass'
   Call growth_grass ()
   
   if (Logging==1) write (File_no(1),*) '    spacial_limitation'
   Call spacial_limitation ()
   
   if (Logging==1) write (File_no(1),*) '    growth_trunc'
   Call growth_trunc ()
   
   if (Logging==1) write (File_no(1),*) '    decompostion'
   Call decomposition (W_sat)
   
!_____________ ANNUALY BIOLOGICAL PROCESSES
IF ( doy==1 ) then
   if (Logging==1) write (File_no(1),*) '    pft_present'
   Call pft_present ()
ENDIF

IF ( doy==Day_in_Year ) then
   
   if (Logging==1) write (File_no(1),*) '    mortality'
   Call mortality ()
   
   if (Logging==1) write (File_no(1),*) '    crown_adjust'
   Call crown_adjust ()
   
   if (Logging==1) write (File_no(1),*) '    crown_shake'
   Call crown_shake ()
   
   !establishment of woody PFTs
   x = sum(prec_RunningRecord(1:Day_in_Year))                  !annual precipitation (mm/year)
   y = sum(tmp_air_RunningRecord(1:Day_in_Year)) / Day_in_Year !annual mean temperature (Celcius)
   if ( x > max(100.0, 20.0*y) ) then                          !from Koppen's criteria (1936)
      
      if (Logging==1) write (File_no(1),*) '    ground_vacant'
       Call ground_vacant ()
       
      if (Logging==1) write (File_no(1),*) '    establish'
       Call establish ()
       
   end if
   
   if (Logging==1) write (File_no(1),*) '    grass priority change'
   Call grass_priority ()
   
   if (Logging==1) write (File_no(1),*) '    fire_regime'
!   if (LON<-20 .or. 60<LON .or. 38.0<LAT) then
      Call fire_regime (W_sat) !Other than African continent
!   else
!      Call fire_regime2 (wind) !African continent
!   endif
   
   if (Logging==1) write (File_no(1),*) '    biome_determine'
   Call biome_determine (W_fi, W_wilt)
   
END IF

!_____________ DAILY UPDATE NET-RADIATION & SOIL-WATER STATUS
   if (Logging==1) write (File_no(1),*) '    net_radiation'
   Call net_radiation (tmp_air(doy), cloud(doy), Albedo_soil0)
   
   if (Logging==1) write (File_no(1),*) '    waterbudget'
   Call waterbudget &
   (tmp_air(doy), tmp_soil(doy,:), prec(doy), wind(doy), W_fi, W_wilt, W_sat)
   
!_____________ MAKE OUTPUT FILES
   IF (Flag_output_write) then
   
   !Daily output (everyday)
   Call output_for_viewer &
   (File_no(2), LAT, LON, ALT, Albedo_soil0, W_fi, W_wilt, W_sat, W_mat, &
   tmp_air(doy), sum(tmp_soil(doy,1:5))/5.0, cloud(doy), prec(doy), humid(doy), wind(doy))
   
   !Daily output (on each specifyed year)
   Call output_climate      (File_no( 3), &
   tmp_air(doy), sum(tmp_soil(doy,1:5))/5.0, cloud(doy), prec(doy), humid(doy), wind(doy))
   Call output_air          (File_no( 4))
   Call output_radiation    (File_no( 5))
   Call output_netradiation (File_no( 6))
   Call output_water        (File_no( 7), W_fi, W_wilt, W_sat)
   Call output_grass        (File_no( 8))
   Call output_lai          (File_no( 9))
   Call output_cflux        (File_no(10))
   Call output_wflux        (File_no(11), prec(doy))
   
   !Monthly output
   if ( Day_of_Month(doy) == Day_in_month(Month(doy)) ) then
      Call output_ld_vertical (File_no(12))
   endif
   
   !Annual output
   if (doy == Day_in_Year) then
      Call output_annual (File_no(13))
      Call output_forest (File_no(14))
   endif
   
   if (doy == 195) then
      Call output_biomass(File_no(15))
   endif
   
   ENDIF
   
END DO !The end of daily loop

!_____________ After Main-simulation-loop procedures
   close (File_no( 1))   !output_log.txt
   close (File_no( 2))   !output.txt
   close (File_no( 3))   !output_climate.txt
   close (File_no( 4))   !output_air.txt
   close (File_no( 5))   !output_radiation.txt
   close (File_no( 6))   !output_netradiation.txt
   close (File_no( 7))   !output_water.txt
   close (File_no( 8))   !output_grass.txt
   close (File_no( 9))   !output_lai.txt
   close (File_no(10))   !output_cflux.txt
   close (File_no(11))   !output_wflux.txt
   close (File_no(12))   !output_ld_vertical.txt
   close (File_no(13))   !output_annual.txt
   close (File_no(14))   !output_forest.txt
   close (File_no(15))   !output_biomass.txt
   
IF ( Flag_spinup_write ) then
   open(File_no(1), file = Fn_spnout)
   Call spinup_out (File_no(1))
   close (File_no(1))
END IF

END SUBROUTINE main_loop
