!*************************************************************************************************
! Start up procedures for selected sites
! (read common parameters and climate data)
!*************************************************************************************************
   Include 'modules.f90'        
   Include 'main.f90'           
   Include 'initialize.f90'     
   Include 'metabolic.f90'      
   Include 'output.f90'         
   Include 'physics.f90'        
   Include 'population_regu.f90'
   Include 'spatial_calc.f90'   
   Include 'etc.f90'            

PROGRAM start_point

!_____________ Set variables
!Namespace
   USE data_structure
   implicit none
   
!Climate data
   real,dimension(Day_in_Year)   ::tmp_air   !2m air temperature (Celcius)
   real,dimension(Day_in_Year)   ::tmp_soil1 !soil temperature   0- 10cm depth (Celcius)
   real,dimension(Day_in_Year)   ::tmp_soil2 !soil temperature  10-200cm depth (Celcius)
   real,dimension(Day_in_Year)   ::tmp_soil3 !soil temperature 200-300cm depth (Celcius)
   real,dimension(Day_in_Year)   ::cloud     !total cloudiness (fraction)
   real,dimension(Day_in_Year)   ::prec      !precipitation (mm day-1)
   real,dimension(Day_in_Year)   ::humid     !specific humidity (kg kg-1)
   real,dimension(Day_in_Year)   ::wind      !wind velocity (m s-1)
   real,dimension(Day_in_Year,30)::tmp_soil  !soil temperature at 10 cm  depth (Celcius)
   
!Location data
   integer Mask         !Land ocean mask (1:land, 0:ocean)
   real    ALT          !altitude (m above MSL)
   real    Albedo_soil0 !albedo, default
   real    W_fi         !filed capacity   (m3/m3, 0.0 -> 1.0)
   real    W_wilt       !wilting point    (m3/m3, 0.0 -> 1.0)
   real    W_sat        !saturate point   (m3/m3, 0.0 -> 1.0)
   real    W_mat        !matrix potential (m, -0.0001 -> -3.0)
   
!Others
   real    LAT, LON      !latitude and logitude for simulate
   integer point         !
   integer i, j, k, rec  !for general usage
   
!_____________ Set location
!  LAT    north:+, south:- (decimalized)
!  LON    east:+, west:-   (decimalized)
    LAT    = 2.97  
    LON    = 102.3 

!set cordinate variables
   point = (90-int(LAT)-1)*360 + int(LON+180) + 1

!_____________ Prepare input data
!Read Parameter files
   open (1, file='parameter.txt', action='READ', status='OLD')
      read ( unit=1, nml=Control)       
      read ( unit=1, nml=PFT_type)      
      read ( unit=1, nml=Respiration)   
      read ( unit=1, nml=Turnover_n)    
      read ( unit=1, nml=Metabolic)     
      read ( unit=1, nml=Assimilation)  
      read ( unit=1, nml=Dynamics)      
      read ( unit=1, nml=Disturbance)   
      read ( unit=1, nml=Soil_resp)     
   close (1)
   
!Read Location data
   open ( File_no(1), file=Fn_location, status='OLD')
   do i=1, point
      read(File_no(1),*) Mask, ALT, Albedo_soil0, W_sat, W_fi, W_mat, W_wilt
   end do
   close( File_no(1) )
   
   !data processing
   if (W_fi   > W_sat ) W_fi   = W_sat
   if (W_wilt > W_sat ) W_wilt = W_sat
   
!Read Climatic data
   open ( File_no(1), file=Fn_climate, status='OLD')
   rec = 0
   do i=1, Day_in_Year
      rec = rec + 1
      read (File_no(1),*) tmp_air(i), tmp_soil1(i), tmp_soil2(i), tmp_soil3(i), &
                          prec(i), cloud(i), wind(i), humid(i)
   enddo
   close (File_no(1))
   
!Interporate soil temperature
   do j=1,Day_in_Year
      do k=1, 20
         tmp_soil(j,k) = (1.0-k/20)      * tmp_soil1(j) + (k/20)      * tmp_soil2(j)
      enddo
      do k=21, 30
         tmp_soil(j,k) = (1.0-(k-20)/10) * tmp_soil2(j) + ((k-20)/10) * tmp_soil3(j)
      enddo
   enddo
   
!_____________ Simulation
   
   !Print location properties
   write (*,*)
   write (*,*) '*********  Coodinate Configurations  *********'
   write (*,*) 'Latitude, Longtitude   :', LAT, LON
   write (*,*) 'Point nomal            :', point
   write (*,*)
   
   !Print soil properties
   write (*,*) '*********  Location properties  *********'
   write (*,*) 'Altitude    :', ALT         
   write (*,*) 'Albedo_soil0:', Albedo_soil0
   write (*,*) 'W_sat       :', W_sat       
   write (*,*) 'W_fi        :', W_fi        
   write (*,*) 'W_wilt      :', W_wilt      
   write (*,*)                              
   
   !Print climate properties
   write (*,*) '*********  Wether statistics (annual mean of the first year)  *********'
   write (*,*) '2m air temperature  (Cecius)  :', sum(tmp_air (:))   / 365.0
   write (*,*) 'soil temperature 1  (Cecius)  :', sum(tmp_soil(:,1)) / 365.0
   write (*,*) 'soil temperature 2  (Cecius)  :', sum(tmp_soil(:,2)) / 365.0
   write (*,*) 'soil temperature 3  (Cecius)  :', sum(tmp_soil(:,3)) / 365.0
   write (*,*) 'precipitation       (mm/year) :', sum(prec    (:))          
   write (*,*) 'cloud               (fraction):', sum(cloud   (:))   / 365.0
   write (*,*) 'humidity            (kg/kg)   :', sum(humid   (:))   / 365.0
   write (*,*) 'wind                (m/s)     :', sum(wind    (:))   / 365.0
   write (*,*) 
  
   !Call simulation loop
   IF (Mask==0 .or. Albedo_soil0<=0.0 .or. W_sat<=0.0 .or. W_fi<=0.0 .or. W_wilt<=0.0 ) then
      write(*,*) 'Error: Invalid location properties'
   ELSE
      write (*,*) '*********  Now simulating  *********'
      Call main_loop ( &
      LAT, LON, &
      tmp_air(:), tmp_soil(:,:), cloud(:), prec(:), humid(:), wind(:), &
      ALT, Albedo_soil0, W_fi, W_wilt, W_sat, W_mat )
      write (*,*) '*********  Done  *********'
   ENDIF
   
   STOP
   
END PROGRAM start_point
